/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See LICENSE.md in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

import { exec } from 'child_process';
import { IActionContext } from 'vscode-azureextensionui';
import { NULL_GUID } from '../../constants';
import { ext } from '../../extensionVariables';
import { registryExpectedContextValues } from '../../tree/registries/registryContextValues';
import { RegistryTreeItemBase } from '../../tree/registries/RegistryTreeItemBase';

export async function logInToDockerCli(context: IActionContext, node?: RegistryTreeItemBase): Promise<void> {
    if (!node) {
        node = await ext.registriesTree.showTreeItemPicker<RegistryTreeItemBase>(registryExpectedContextValues.all.registry, context);
    }

    let creds = await node.getDockerCliCredentials();
    let auth: { username?: string, password?: string, token?: string } = creds.auth || {};
    let username: string | undefined;
    let password: string | undefined;
    if (auth.token) {
        username = NULL_GUID;
        password = auth.token;
    } else if (auth.password) {
        username = auth.username;
        password = auth.password;
    }

    if (!username || !password) {
        ext.outputChannel.appendLine(`WARNING: Skipping login for "${creds.registryPath}" because it does not require authentication.`)
    } else {
        await new Promise((resolve, reject) => {
            const dockerLoginCmd = `docker login ${creds.registryPath} --username ${username} --password-stdin`;
            let childProcess = exec(dockerLoginCmd, (err, stdout, stderr) => {
                ext.outputChannel.appendLine(dockerLoginCmd);
                ext.outputChannel.append(stdout);
                ext.outputChannel.append(stderr);
                if (err && err.message.match(/error storing credentials.*The stub received bad data/)) {
                    // Temporary work-around for this error- same as Azure CLI
                    // See https://github.com/Azure/azure-cli/issues/4843
                    reject(new Error(`In order to log in to the Docker CLI using tokens, you currently need to go to \nOpen your Docker config file and remove "credsStore": "wincred" from the config.json file, then try again. \nDoing this will disable wincred and cause Docker to store credentials directly in the .docker/config.json file. All registries that are currently logged in will be effectly logged out.`));
                } else if (err) {
                    reject(err);
                } else if (stderr) {
                    reject(stderr);
                } else {
                    resolve();
                }
            });

            childProcess.stdin.write(password); // Prevents insecure password error
            childProcess.stdin.end();
        });
    }

    ext.outputChannel.show();
}
