/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See LICENSE.md in the project root for license information.
 *--------------------------------------------------------------------------------------------*/

import { RequestPromiseOptions } from "request-promise-native";
import { AzExtParentTreeItem, AzExtTreeItem, IActionContext } from "vscode-azureextensionui";
import { PAGE_SIZE } from "../../../constants";
import { ext } from "../../../extensionVariables";
import { nonNullProp } from "../../../utils/nonNull";
import { getNextLinkFromHeaders, registryRequest } from "../../../utils/registryRequestUtils";
import { getIconPath, IconPath } from "../../IconPath";
import { ICachedRegistryProvider } from "../ICachedRegistryProvider";
import { IRegistryProviderTreeItem } from "../IRegistryProviderTreeItem";
import { getRegistryContextValue, registryProviderSuffix } from "../registryContextValues";
import { getRegistryPassword } from "../registryPasswords";
import { GitLabProjectTreeItem } from "./GitLabProjectTreeItem";

export class GitLabAccountTreeItem extends AzExtParentTreeItem implements IRegistryProviderTreeItem {
    public label: string = 'GitLab';
    public childTypeLabel: string = 'project';
    public baseUrl: string = 'https://gitlab.com/';
    public cachedProvider: ICachedRegistryProvider;

    private _token?: string;
    private _nextLink?: string;

    public constructor(parent: AzExtParentTreeItem, provider: ICachedRegistryProvider) {
        super(parent);
        this.cachedProvider = provider;
    }

    public get contextValue(): string {
        return getRegistryContextValue(this, registryProviderSuffix);
    }

    public get description(): string | undefined {
        return ext.registriesRoot.hasMultiplesOfProvider(this.cachedProvider) ? this.username : undefined;
    }

    public get iconPath(): IconPath {
        return getIconPath('gitLab');
    }

    public get id(): string {
        return this.cachedProvider.id + this.username;
    }

    public get username(): string {
        return nonNullProp(this.cachedProvider, 'username');
    }

    public async getPassword(): Promise<string> {
        return await getRegistryPassword(this.cachedProvider);
    }

    public async loadMoreChildrenImpl(clearCache: boolean, _context: IActionContext): Promise<AzExtTreeItem[]> {
        if (clearCache) {
            this._nextLink = undefined;
            await this.refreshToken();
        }

        const url: string = this._nextLink || `api/v4/projects?per_page=${PAGE_SIZE}&simple=true&membership=true`;
        let response = await registryRequest<IProject[]>(this, 'GET', url);
        this._nextLink = getNextLinkFromHeaders(response);
        return this.createTreeItemsWithErrorHandling(
            response.body,
            'invalidGitLabProject',
            n => new GitLabProjectTreeItem(this, n.id.toString(), n.path_with_namespace),
            n => n.path_with_namespace
        );
    }

    public hasMoreChildrenImpl(): boolean {
        return !!this._nextLink;
    }

    public async addAuth(options: RequestPromiseOptions): Promise<void> {
        if (this._token) {
            options.auth = {
                bearer: this._token
            }
        }
    }

    private async refreshToken(): Promise<void> {
        this._token = undefined;
        const options = {
            form: {
                grant_type: "password",
                username: this.username,
                password: await this.getPassword()
            }
        };

        const response = await registryRequest<IToken>(this, 'POST', 'oauth/token', options);
        this._token = response.body.access_token;
    }
}

interface IProject {
    id: number;
    path_with_namespace: string;
}

interface IToken {
    access_token: string
}
